#!/usr/bin/perl -w

use CGI qw/:standard *blockquote *table *Tr *div/;
# http://www.geocities.co.jp/SiliconValley-Sunnyvale/6128/perl/cgipm.html
# http://www.tohoho-web.com/wwwperl1.htm

sub ln { "\n"; }

sub image {
    my ($e, @align) = @_;
    img({-src=>"$e.png", -alt=>"$e", @align});
    #"<img src=\"$e.png\" alt=\"$e\">";
}

$NATURAL = '<img src="natural.png">';  # without align

param('a:min', -5);
param('a:max', 0);
#param('aa:min', 1); # unused
param('aa:max', 50);
param('b:min', -100);
param('b:max', 100);
param('x:min', -50);
param('x:max', 50);

my $dim_min     = 1;
my $dim_default = 3;
my $dim_max     = 7;
my $dim = param('dim');
unless (defined($dim) && $dim_min <= $dim && $dim <= $dim_max) {
    $dim = $dim_default;
    param('dim', $dim);
}

sub change_dim {
    $dim = int(param('newdim'));
    unless ($dim_min <= $dim && $dim <= $dim_max) {
	$dim = $dim_default;
    }
    param('dim', $dim);
}

sub print_header {
print
    header(-expires=>'+1m'), # cache expires in 1min
    start_html(-title=> 'Quadratic L#-Convex',
	       -head=>[meta( { '-http_equiv' => 'Content-style-type',
			       '-content'    => 'text/html' } ),
		       meta( { '-http_equiv' => 'Content-script-type',
			       '-content'    => 'text/javascript' } )],
	       -onload=>'init()'),
    ln, <<EOF,
<script type="text/javascript">
<!--
var a;
var out_range;
function init() {
    a = new Array($dim);
    for (i=0; i<$dim; i++) {
	a[i] = new Array($dim);
	for (j=0; j<$dim; j++) {
	    a[i][j] = document.p2.elements['a:'+i+':'+j];
	}
    }
    check();
}
function range_check(e, min, max) {
    e.value = ~~e.value;  // parseInt(e.value);
    if (min <= e.value && e.value <= max) {
	e.style.backgroundColor = 'white';
    } else {
	e.style.backgroundColor = 'pink';
	out_range = true;
    }
}
function check() {
    var p2 = document.p2;
    var dummy = 0;
    var list = {'b':dummy, 'x':dummy};
    out_range = false;

    for (var v in list) {
	var min = ~~p2.elements[v+':min'].value - 0;
	var max = ~~p2.elements[v+':max'].value - 0;
	for (i=0; i<$dim; i++) {
	    range_check(p2.elements[v+':'+i], min, max);
	}
    }
    var min = ~~p2.elements['a:min'].value - 0;
    var max = ~~p2.elements['a:max'].value - 0;
    for (i=0; i<$dim; i++) {
	for (j=0; j<i; j++) {
	    range_check(a[i][j], min, max);
	}
    }
    max = ~~p2.elements['aa:max'].value - 0;
    for (i=0; i<$dim; i++) {
	var sum = 0;
	for (j=0;   j<i;    j++) { sum += a[i][j].value - 0; }
	for (j=i+1; j<$dim; j++) { sum += a[j][i].value - 0; }
	range_check(a[i][i], -sum, max);
    }
    if (out_range) {
	p2.Minimize.disabled = true;
    } else {
	p2.Minimize.disabled = false;
    }

    for (i=0; i<$dim; i++) {
	for (j=i+1; j<$dim; j++) {
	    a[i][j].value = a[j][i].value;
	}
    }
}

function popup_window(form) {
    obj = window.open("", "popup", "width=400, height=400, scrollbars=yes, menubar=no, location=no, directories=no");
    form.target = "popup";
    form.action = "./odicon.cgi";
    obj.focus();
    form.submit();
}
// http://www.phppro.jp/qa/609
// -->
</script>
EOF
    a({href=>"./"},'DCP HOME'), ln,
    h1('Quadratic L<img src="natural.png" width=14>-Convex Minimization'), ln,
    start_blockquote, ln;
}

sub set_param {
    my ($var, $min, $max) = @_;
    my $val = $min + int(rand($max-$min+1));
    param($var, $val);
}

sub init_param {
    my ($i, $j);

    foreach $e ('b', 'x') {
	my $min = param("$e:min");
	my $max = param("$e:max");
	for ($i=0; $i<$dim; $i++) {
	    set_param("$e:$i", $min, $max);
	}
    }

    foreach $e ('a') {
	my $min = param("$e:min");
	my $max = param("$e:max");
	for ($i=0; $i<$dim; $i++) {
	    for ($j=0; $j<$i; $j++) {
		set_param("$e:$i:$j", $min, $max);
	    }
	}

	$max = param("$e$e:max");
	for ($i=0; $i<$dim; $i++) {
	    my $sum = 0;
	    for ($j=0;    $j<$i;   $j++) { $sum += param("$e:$i:$j"); }
	    for ($j=$i+1; $j<$dim; $j++) { $sum += param("$e:$j:$i"); }
	    set_param("$e:$i:$i", -$sum, $max);
	}
    }
}

sub print_html_table {
    print 'For a symmetric matrix ', image('quadM-A', -align=>'middle'),
    ' with ', image('quadL-defA',  -align=>'middle'), ln;
    print 'the associated quadratic form  ',
    image('quadform', -align=>'middle'), 
    " is an L$NATURAL-convex function [1,2]. ", ln;
    print p, "Here we consider a quadratic L$NATURAL-convex function:", ln;
    print br, image('objfunc-quadM'),".".ln;

    print p, "In this web application, you can choose the dimension n from 1 to $dim_max.", ln;

    print start_form, ln;
    @index = (1..$dim_max);
    print p, 'dimension n = ', popup_menu(-name=>'newdim',-values=>[@index],-default=>$dim);
    print submit('Change'), ln, end_form, ln;

    print p, 'You can input parameters a', Sub(em('ij')),
    ' and b', Sub(em('i')),
    ' in the ranges ', param('a:min'), ' &le; a', Sub(em('ij')), '&le; ', param('a:max'),
    image('quadL-ineqj',  -align=>'middle'),
    ', 0 &lt; a', Sub(em('ii')), '&le; ', param('aa:max'),
    ' and ', param('b:min'), ' &le; b', Sub(em('i')), '&le; ', param('b:max'),'.',
    ' If the input does not satisfy the condition ', image('quadL-defA',  -align=>'middle'),
    'the unsatisfactory condition turns red.', ln;
    #', b', Sub(em('i')),', c', Sub(em('i')),
    print p, 'You can also input an initial solution x.', ln;
  
    print start_form(-name=>'p2',
		     -action=>'javascript:popup_window(document.p2)'),
    hidden('dim'), ln;

    foreach $e ('a') {
	print br, hidden("$e:min"), hidden("$e:max"), hidden("$e$e:max"),
	start_table({-border=>'1'}), ln,
	Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'},
				  [ 'A', 1 .. $dim ])]);
	for ($i=0; $i<$dim; $i++) {
	    my @td = ($i+1);
	    for ($j=0; $j<$dim; $j++) {
		my $element = '';
		if ($i >= $j) {
		    $element .= textfield(-name=>"$e:$i:$j", -size=>3,
					  -default=>param("$e:$i:$j"),
					  -onMouseOut=>"check()",
					  -onBlur=>"check()");
		} else {
		    $element .= textfield(-name=>"$e:$i:$j", -size=>3,
					  -disabled);
		}
		push @td, $element;
	    }
	    print Tr({-align=>'right'}, [td([@td])]), ln;
	}
	print end_table, ln;
    }

    print br, start_table({-border=>'1'}), ln;
    print Tr({-align=>'right'}, [td({-width=>int(100/(1+$dim)).'%'},
				    [ '', 1 .. $dim ])]);
    foreach $e ('b', 'x') {
	my @td = ($e);
	for ($j=0; $j<$dim; $j++) {
	    push @td, textfield(-name=>"$e:$j", -size=>3,
				-default=>param("$e:$j"),
				-onMouseOut=>"check()", -OnBlur=>"check()");
	}
	print Tr({-align=>'right'}, [td([@td])]),
	hidden("$e:min"), hidden("$e:max"), ln;
    }
    print end_table, ln;

    print p, '(Note: "Reset" button resets your inputs to default values.)', ln;
    #print p, start_table(),Tr(td({-id=>'message'}, '')),end_table;
    print p,
    submit(-name=>'Minimize', -disabled), ln,
    reset(-name=>'Reset', -onMouseOut=>"check()", -OnBlur=>"check()"), ln,
    end_form, ln;
}

sub print_reference {
    print <<EOF;
<hr width="70%">
<p> This web application minimizes f(x) using
<a href="http://www.misojiro.t.u-tokyo.ac.jp/~tutimura/odicon/index.en.html">ODICON</a>. 

<p> [1] K. Murota (2001): "Discrete Convex Analysis---An Introduction (in Japanese),"
Kyoritsu Publishing Company, Tokyo. Section 5.2.

<p> [2] K. Murota (2003):
"Discrete Convex Analysis," SIAM. Section 7.3.
EOF
# <p> [3] K. Murota,
# Recent developments in discrete convex analysis,
# in: W. Cook, L. Lovasz and J. Vygen, eds.,
# Research Trends in Combinatorial Optimization, Bonn 2008,
# Springer-Verlag, Berlin, (2009), Chapter 11, 219--260.
}

change_dim() if param('Change');
init_param();
print_header();
print_html_table();
print_reference();

print end_blockquote, end_html, ln;
